#include "Tools.h"

#pragma comment(lib, "..\\Bass64.lib")
#include "bass.h"

const wchar_t   *$BASSMUSICEXT = L".mo3.it.xm.s3m.mtm.mod.umx.";
const wchar_t   *$BASSAUDIOEXT = L"mp3.ogg.wav.mo3.it.xm.s3m.mtm.mod.umx.";

#define STREAM  0
#define MUSIC   1

struct TAUDIO {
    WCHAR    AudioFile[MAX_PATH];

    HSTREAM  Channel;       // The playing audio channel
    WORD Lpeak;             // The left audio channel peak value (while in play mode).
    WORD Rpeak;             // The right audio channel peak value (while in play mode).
    LONG_PTR FFTdata;       // DWORD pointer to the FFT() AS SINGLE array.
    LONG_PTR FFTsize;       // Size of the FFT array.

    LONG_PTR WIMdata;       // DWORD pointer to the wave MM_WIM_DATA.
    DWORD MediaLength;      // Media length.
    DWORD MediaPos;         // Media pos.

    long MediaType;         // MUSIC, STREAM, etc.
    long InitDone;          // BassInit flag.
};

TAUDIO ta;

LONG_PTR BassChannelGetData () {
    static float fft[256];
    memset(&fft[0], 0, sizeof(float) * 256);
    if (ta.Channel) { long nRet = BASS_ChannelGetData(ta.Channel, &fft[0], BASS_DATA_FFT512); }
    return (LONG_PTR) &fft[0];
}

long BassFree () {
    long nRet = 0;
    if (ta.InitDone) { ta.InitDone = 0; nRet = BASS_Free(); }
    return nRet;
}

long BassCloseChannel () {
    long nRet = 0;
    if (ta.Channel) {
       if (ta.MediaType == MUSIC) {
          nRet = BASS_MusicFree(ta.Channel); }
       else {
          nRet = BASS_StreamFree(ta.Channel);
       }
       ta.Channel = 0; ta.MediaType = 0;
    }
    return nRet;
}

long BassInit (IN HWND hWnd) {
    if (ta.InitDone == 0) { ta.InitDone = BASS_Init(1, 44100, 0, hWnd, 0); }
    return ta.InitDone;
}

void CALLBACK BassStreamEllapsed(IN HSYNC handle, IN DWORD channel, IN DWORD data, void *user) {
    BassCloseChannel();
}

long BassChannelPlay(IN HWND hWnd) {
    long nRet = 0;

    wstring sExt;

    BassCloseChannel();

    if (BassInit(hWnd)) {
        long So = INSTR(-1, ta.AudioFile, $DOT);
        if (So) { sExt = LCASE$(MID$(ta.AudioFile, So)); sExt.append($DOT); So = INSTR(0, $BASSMUSICEXT, sExt); }
        if (So) {
            ta.MediaType = MUSIC;
            ta.Channel = BASS_MusicLoad(0, ta.AudioFile, 0, 0, BASS_STREAM_PRESCAN | BASS_STREAM_AUTOFREE, 0);
            }
        else {
            ta.MediaType = STREAM;
            ta.Channel = BASS_StreamCreateFile(0, ta.AudioFile, 0, 0, BASS_STREAM_AUTOFREE);
        }

        if (ta.Channel) {
            BASS_ChannelSetSync(ta.Channel, BASS_SYNC_END | BASS_SYNC_ONETIME, 0, &BassStreamEllapsed, 0);
            nRet = BASS_ChannelPlay(ta.Channel, 0);
            HSTREAM nScanChannel = BASS_StreamCreateFile(0, ta.AudioFile, 0, 0, BASS_STREAM_DECODE | BASS_UNICODE);
            }
        else {
            BassCloseChannel();
        }
    }
    return nRet;
}

long BassStreamFree() {
    long nRet = 0;
    if (ta.Channel) {
        if (ta.MediaType == MUSIC) {
            nRet = BASS_MusicFree(ta.Channel); }
        else {
            nRet = BASS_StreamFree(ta.Channel);
       }
       ta.Channel = 0; ta.MediaType = 0;
    }
    return nRet;
}

WORD SolvePeak(IN WORD nValue, IN WORD nTotal) {
    WORD nRet = 0;
    if (nTotal) { nRet = (nValue * 100) / nTotal; }
    return nRet;
}

LONG_PTR BassChannelGetWimData (IN HSTREAM nChannel) {
    static float fft[256];
    memset(&fft[0], 0, sizeof(float) * 256);
    if (nChannel) { long nRet = BASS_ChannelGetData(nChannel, &fft[0], 1024); }
    return (LONG_PTR) &fft[0];
}
